# -*- coding: utf-8 -*-

"""
This module provides functions for searching the courses
in command line interface by pysaint

"""
from __future__ import absolute_import

import click

from pysaint.saint import Saint


@click.group()
def main():
    """
    TODO: Make CLI Interface
    :return:
    """
    pass
